@@ -22,10 +22,6 @@
 #include <thunar-vfs/thunar-vfs.h>
 #include <libexif/exif-data.h>
 
-#ifdef HAVE_XFCONF
-#include <xfconf/xfconf.h>
-#endif
-
 #include "navigator.h"
 #include "picture_viewer.h"
 #include "main_window.h"
@@ -57,7 +53,7 @@ gboolean start_slideshow = FALSE;
 
 static GOptionEntry entries[] =
 {
-    {    "version", 'v', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &version,
+    {    "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &version,
         N_("Version information"),
         NULL
     },
@@ -221,6 +217,7 @@ int main(int argc, char **argv)
     GdkColor *bg_color = NULL;
     GError *cli_error = NULL;
     gint n;
+    gchar *program = NULL;
 
     #ifdef ENABLE_NLS
     bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -245,14 +242,24 @@ int main(int argc, char **argv)
     }
 
     
-#ifdef HAVE_XFCONF
-    xfconf_init(NULL);
-#endif
-
     thunar_vfs_init();
 
     mime_dbase = thunar_vfs_mime_database_get_default();
 
+    program = g_find_program_in_path ("xfconf-query");
+    if (G_LIKELY (program != NULL))
+    {
+        rstto_has_xfconf_query = TRUE;
+        g_free (program);
+    }
+
+    program = g_find_program_in_path ("gconftool");
+    if (G_LIKELY (program != NULL))
+    {
+        rstto_has_gconftool = TRUE;
+        g_free (program);
+    }
+
 
     gtk_window_set_default_icon_name("ristretto");
     xfce_rc = xfce_rc_config_open(XFCE_RESOURCE_CONFIG, "ristretto/ristrettorc", FALSE);
@@ -265,7 +272,7 @@ int main(int argc, char **argv)
     gint slideshow_timeout = xfce_rc_read_int_entry(xfce_rc, "SlideShowTimeout", 5000);
     gboolean slideshow_hide_thumbnail = xfce_rc_read_bool_entry(xfce_rc, "SlideShowHideThumbnail", TRUE);
     gint max_cache = xfce_rc_read_int_entry(xfce_rc, "MaxImagesCacheSize", 64);
-    gboolean preload_during_slideshow = xfce_rc_read_bool_entry (xfce_rc, "PreloadDuringSlideShow", FALSE);
+    gboolean preload_imgs = xfce_rc_read_bool_entry (xfce_rc, "PreloadImgs", FALSE);
     gboolean override_bg_color = xfce_rc_read_bool_entry (xfce_rc, "OverrideBgColor", FALSE);
     gboolean scale_to_100 = xfce_rc_read_bool_entry (xfce_rc, "ScaleTo100", FALSE);
 
@@ -284,7 +291,7 @@ int main(int argc, char **argv)
 
     RsttoNavigator *navigator = rstto_main_window_get_navigator(RSTTO_MAIN_WINDOW(window));
 
-    navigator->preload = preload_during_slideshow;
+    navigator->preload = preload_imgs;
 
     rstto_main_window_set_max_cache_size(RSTTO_MAIN_WINDOW(window), max_cache);
     rstto_main_window_set_slideshow_timeout(RSTTO_MAIN_WINDOW(window), (gdouble)slideshow_timeout);
@@ -346,7 +353,7 @@ int main(int argc, char **argv)
 
     xfce_rc_write_bool_entry(xfce_rc, "ShowToolBar", rstto_main_window_get_show_toolbar(RSTTO_MAIN_WINDOW(window)));
     xfce_rc_write_bool_entry(xfce_rc, "ScaleTo100", rstto_main_window_get_scale_to_100(RSTTO_MAIN_WINDOW(window)));
-    xfce_rc_write_bool_entry(xfce_rc, "PreloadDuringSlideShow", navigator->preload);
+    xfce_rc_write_bool_entry(xfce_rc, "PreloadImgs", navigator->preload);
     xfce_rc_write_bool_entry(xfce_rc, "ShowThumbnailViewer", rstto_main_window_get_show_thumbnail_viewer(RSTTO_MAIN_WINDOW(window)));
     if (bg_color)
     {
@@ -372,9 +379,7 @@ int main(int argc, char **argv)
     xfce_rc_flush(xfce_rc);
     xfce_rc_close(xfce_rc);
     gtk_widget_unref(window);
-#ifdef HAVE_XFCONF
-    xfconf_shutdown();
-#endif
+
     return 0;
 }
 
