@@ -27,14 +27,11 @@
 #include <libexif/exif-data.h>
 #include <dbus/dbus-glib.h>
 
-#ifdef HAVE_XFCONF
-#include <xfconf/xfconf.h>
-#endif
-
 #include "navigator.h"
 #include "thumbnail_bar.h"
 #include "picture_viewer.h"
 #include "main_window.h"
+#include "save_dialog.h"
 
 #define XFDESKTOP_SELECTION_FMT "XFDESKTOP_SELECTION_%d"
 
@@ -424,50 +421,22 @@ rstto_main_window_init(RsttoMainWindow *window)
     window->priv->menus.view.menu_item_set_wallpaper = gtk_image_menu_item_new_with_mnemonic(_("_Set as wallpaper"));
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(window->priv->menus.view.menu_item_set_wallpaper), wallpaper_image);
 
-    /** FIXME: HACK */
-#ifdef HAVE_XFCONF 
 #ifdef WITH_DESKTOP_WALLPAPER
     /** Set xfce-desktop as default when support has been compiled in */
     /* Check if xfdesktop is running */
     {
-        Atom actual_type;
-        int actual_format;
-        unsigned long n_items;
-        unsigned long bytes_after;
-        unsigned char *prop;
         gchar selection_name[100];
 
         GdkScreen *gdk_screen = gdk_screen_get_default();
         gint xscreen = gdk_screen_get_number(gdk_screen);
 
-
         g_snprintf(selection_name, 100, XFDESKTOP_SELECTION_FMT, xscreen);
 
-        Window root_window = GDK_ROOT_WINDOW();
-        Atom xfce_desktop_atom = XInternAtom (gdk_display, "XFCE_DESKTOP_WINDOW", False);
         Atom xfce_selection_atom = XInternAtom (gdk_display, selection_name, False);
-        XGetWindowProperty (gdk_display, 
-                            root_window,
-                            xfce_desktop_atom,
-                            0,
-                            1,
-                            False,
-                            AnyPropertyType,
-                            &actual_type,
-                            &actual_format,
-                            &n_items,
-                            &bytes_after,
-                            &prop);
-        if (n_items > 0)
+        if((XGetSelectionOwner(GDK_DISPLAY(), xfce_selection_atom)))
         {
-            /* TODO: check XID */
-            /* Window xid = (Window) prop[1]; */
-            if(XGetSelectionOwner(gdk_display, xfce_selection_atom) == prop[1] != 0)
+            if (rstto_has_xfconf_query)
                 window->priv->settings.desktop = RSTTO_DESKTOP_XFCE;
-            else
-            {
-                g_debug("xfdesktop is not running");
-            }
         }
         else
         {
@@ -475,8 +444,6 @@ rstto_main_window_init(RsttoMainWindow *window)
         }
     }
 #endif
-#endif
-
 
     gtk_widget_add_accelerator(window->priv->menus.view.menu_item_fullscreen, "activate", accel_group, GDK_F11, 0,GTK_ACCEL_VISIBLE);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(window->priv->menus.view.menu_item_show_toolbar), TRUE);
@@ -915,8 +882,34 @@ static void
 rstto_main_window_dispose(GObject *object)
 {
     RsttoMainWindow *window = RSTTO_MAIN_WINDOW(object);
-    if (window->priv->navigator)
+    RsttoNavigator  *navigator = window->priv->navigator;
+    if (navigator)
     {
+#if 0
+        GList *iter, *modified_files = NULL;
+        
+        for (iter = navigator->file_list; iter != NULL; iter = iter->next)
+        {
+            if (rstto_navigator_entry_get_orientation_changed ((RsttoNavigatorEntry *)iter->data))
+            {
+                modified_files = g_list_append (modified_files, iter->data);
+            }
+        }
+
+        if (modified_files)
+        {
+            GtkWidget *dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL, _("The orientation of one or more images has been modified, do you want to save the changes?")); 
+            if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+            {
+                gtk_widget_hide(dialog);
+                GtkWidget *save_dialog = rstto_save_dialog_new (GTK_WINDOW(window), modified_files);
+                if (gtk_dialog_run (GTK_DIALOG(save_dialog)) == GTK_RESPONSE_OK)
+                {
+                    
+                }
+            }
+        }
+#endif
         g_object_unref(window->priv->navigator);
         window->priv->navigator = NULL;
     }
@@ -1188,46 +1181,54 @@ cb_rstto_main_window_set_wallpaper(GtkWidget *widget, RsttoMainWindow *window)
     ThunarVfsInfo *info = rstto_navigator_entry_get_info(entry);
     gchar *path = thunar_vfs_path_dup_string(info->path);
 
+    GdkScreen *gdk_screen = gdk_screen_get_default();
+    gint screen = gdk_screen_get_number(gdk_screen);
+    gint monitor = gdk_screen_get_monitor_at_window(gdk_screen, GTK_WIDGET(window)->window);
+
+    gchar *image_path_prop = NULL;
+    gchar *image_show_prop = NULL;
+    gchar *image_style_prop = NULL;
+    gchar *command = NULL;
 
     switch (window->priv->settings.desktop)
     {
-#ifdef HAVE_XFCONF
         case RSTTO_DESKTOP_XFCE:
             {
 
-                XfconfChannel *xfdesktop_channel = xfconf_channel_new("xfce4-desktop");
-
                 /*
                  * Retrieve the screen and monitor number where the main ristretto window is running,
                  * set the wallpaper there.
                  */
-                GdkScreen *gdk_screen = gdk_screen_get_default();
-                gint screen = gdk_screen_get_number(gdk_screen);
-                gint monitor = gdk_screen_get_monitor_at_window(gdk_screen, GTK_WIDGET(window)->window);
+                image_path_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-path", screen, monitor);
+                image_show_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-show", screen, monitor);
+                image_style_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-style", screen, monitor);
+
+                command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t string -s %s", image_path_prop, path);
+                g_spawn_command_line_async (command, NULL);
+                g_free (command);
+
+                command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t bool -s true", image_show_prop);
+                g_spawn_command_line_async (command, NULL);
+                g_free (command);
+
+                command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t int -s 4", image_style_prop);
+                g_spawn_command_line_async (command, NULL);
+                g_free (command);
 
-                gchar *image_path_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-path", screen, monitor);
-                gchar *image_show_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-show", screen, monitor);
-                gchar *image_style_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-style", screen, monitor);
-                if(xfconf_channel_set_string(xfdesktop_channel, image_path_prop, path) == TRUE)
-                {
-                    xfconf_channel_set_bool(xfdesktop_channel, image_show_prop, TRUE);
-                    xfconf_channel_set_int(xfdesktop_channel, image_style_prop, 4);
-                }
-                else
-                {
-                    /** FAILED */
-                }
-                g_free(image_path_prop);
-                g_free(image_show_prop);
-                g_object_unref(xfdesktop_channel);
             }
             break;
-#endif
         default:
             g_debug("not supported");
             break;
     }
-    g_free(path);
+    if (image_path_prop)
+        g_free(image_path_prop);
+    if (image_show_prop)
+        g_free(image_show_prop);
+    if (image_style_prop)
+        g_free(image_style_prop);
+    if (path)
+        g_free(path);
 }
 #endif
 
@@ -1412,17 +1413,17 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), notebook,  TRUE, TRUE, 0);
 
 /** Add notebook pages */
-    behaviour_main_vbox = gtk_vbox_new(FALSE, 0);
-    behaviour_main_lbl = gtk_label_new(_("Behaviour"));
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), behaviour_main_vbox, behaviour_main_lbl);
+    display_main_vbox = gtk_vbox_new(FALSE, 0);
+    display_main_lbl = gtk_label_new(_("Display"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), display_main_vbox, display_main_lbl);
 
     slideshow_main_vbox = gtk_vbox_new(FALSE, 0);
     slideshow_main_lbl = gtk_label_new(_("Slideshow"));
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), slideshow_main_vbox, slideshow_main_lbl);
 
-    display_main_vbox = gtk_vbox_new(FALSE, 0);
-    display_main_lbl = gtk_label_new(_("Display"));
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), display_main_vbox, display_main_lbl);
+    behaviour_main_vbox = gtk_vbox_new(FALSE, 0);
+    behaviour_main_lbl = gtk_label_new(_("Behaviour"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), behaviour_main_vbox, behaviour_main_lbl);
 
 /** Add content for behaviour page */
     resize_to_content_vbox = gtk_vbox_new(FALSE, 0);
@@ -1469,11 +1470,14 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     cache_label = gtk_label_new(_("Cache size:"));
     cache_mb_label = gtk_label_new(_("MB"));
 
+    GtkWidget *preload_check = gtk_check_button_new_with_mnemonic(_("_Preload images (GUI more responsive but more CPU-intensive)"));
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preload_check), window->priv->navigator->preload);
+
     gtk_box_pack_start(GTK_BOX(cache_hbox), cache_label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(cache_hbox), cache_spin_button, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(cache_hbox), cache_mb_label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(cache_vbox), cache_hbox, FALSE, FALSE, 0);
-
+    gtk_box_pack_start(GTK_BOX(cache_vbox), preload_check, FALSE, FALSE, 0);
 
     gtk_container_set_border_width (GTK_CONTAINER (bg_color_frame), 8);
     gtk_container_set_border_width (GTK_CONTAINER (cache_frame), 8);
@@ -1481,55 +1485,40 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     gtk_box_pack_start(GTK_BOX(display_main_vbox), bg_color_frame, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(display_main_vbox), cache_frame, FALSE, TRUE, 0);
 
+
 /** Add content for slideshow page */
     GtkWidget *slideshow_vbox = gtk_vbox_new(FALSE, 0);
     GtkWidget *slideshow_frame = xfce_create_framebox_with_content (_("Timeout"), slideshow_vbox);
 
-    GtkWidget *preload_vbox = gtk_vbox_new(FALSE, 0);
-    GtkWidget *preload_frame = xfce_create_framebox_with_content (_("Preload"), preload_vbox);
-
     GtkWidget *thumbnails_vbox = gtk_vbox_new(FALSE, 0);
     GtkWidget *thumbnails_frame = xfce_create_framebox_with_content (_("Thumbnails"), thumbnails_vbox);
 
     gtk_container_set_border_width (GTK_CONTAINER (slideshow_frame), 8);
     gtk_container_set_border_width (GTK_CONTAINER (thumbnails_frame), 8);
-    gtk_container_set_border_width (GTK_CONTAINER (preload_frame), 8);
 
     GtkWidget *slideshow_lbl = gtk_label_new(_("The time period an individual image is displayed during a slideshow\n(in seconds)"));
     GtkWidget *slideshow_hscale = gtk_hscale_new_with_range(1, 60, 1);
 
-
-    GtkWidget *preload_lbl = gtk_label_new(_("Preload images during slideshow\n(uses more memory)"));
-    GtkWidget *preload_check = gtk_check_button_new_with_mnemonic(_("_Preload images"));
-
     GtkWidget *thumbnails_lbl = gtk_label_new(_("Hide the thumbnails bar during slideshow"));
     GtkWidget *thumbnails_check = gtk_check_button_new_with_mnemonic(_("_Hide thumbnails"));
 
     gtk_misc_set_alignment(GTK_MISC(slideshow_lbl), 0, 0.5);
-    gtk_misc_set_alignment(GTK_MISC(preload_lbl), 0, 0.5);
     gtk_misc_set_alignment(GTK_MISC(thumbnails_lbl), 0, 0.5);
 
     gtk_misc_set_padding(GTK_MISC(slideshow_lbl), 2, 2);
-    gtk_misc_set_padding(GTK_MISC(preload_lbl), 2, 2);
     gtk_misc_set_padding(GTK_MISC(thumbnails_lbl), 2, 2);
     
     gtk_range_set_value(GTK_RANGE(slideshow_hscale), window->priv->settings.slideshow_timeout / 1000);
 
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preload_check), window->priv->navigator->preload);
-
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(thumbnails_check), window->priv->settings.slideshow_hide_thumbnail);
 
     gtk_box_pack_start(GTK_BOX(slideshow_vbox), slideshow_lbl, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(slideshow_vbox), slideshow_hscale, FALSE, TRUE, 0);
 
-    gtk_box_pack_start(GTK_BOX(preload_vbox), preload_lbl, TRUE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(preload_vbox), preload_check, FALSE, TRUE, 0);
-
     gtk_box_pack_start(GTK_BOX(thumbnails_vbox), thumbnails_lbl, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(thumbnails_vbox), thumbnails_check, FALSE, TRUE, 0);
 
     gtk_box_pack_start(GTK_BOX(slideshow_main_vbox), slideshow_frame, FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(slideshow_main_vbox), preload_frame, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(slideshow_main_vbox), thumbnails_frame, FALSE, TRUE, 0);
 
     gtk_widget_show_all(notebook);
@@ -1591,7 +1580,7 @@ cb_rstto_main_window_about(GtkWidget *widget, RsttoMainWindow *window)
     gtk_about_dialog_set_license((GtkAboutDialog *)about_dialog,
         xfce_get_license_text(XFCE_LICENSE_TEXT_GPL));
     gtk_about_dialog_set_copyright((GtkAboutDialog *)about_dialog,
-        "Copyright \302\251 2006-2007 Stephan Arts");
+        "Copyright \302\251 2006-2008 Stephan Arts");
 
     gtk_dialog_run(GTK_DIALOG(about_dialog));
 
@@ -1762,6 +1751,37 @@ cb_rstto_main_window_close(GtkWidget *widget, RsttoMainWindow *window)
 static void
 cb_rstto_main_window_close_all(GtkWidget *widget, RsttoMainWindow *window)
 {
+#if 0
+    RsttoNavigator  *navigator = window->priv->navigator;
+
+    if (navigator)
+    {
+        GList *iter, *modified_files = NULL;
+        
+        for (iter = navigator->file_list; iter != NULL; iter = iter->next)
+        {
+            if (rstto_navigator_entry_get_orientation_changed ((RsttoNavigatorEntry *)iter->data))
+            {
+                modified_files = g_list_append (modified_files, iter->data);
+            }
+        }
+
+        if (modified_files)
+        {
+            GtkWidget *dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL, _("The orientation of one or more images has been modified, do you want to save the changes?")); 
+            if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+            {
+                gtk_widget_hide(dialog);
+                GtkWidget *save_dialog = rstto_save_dialog_new (GTK_WINDOW(window), modified_files);
+                if (gtk_dialog_run (GTK_DIALOG(save_dialog)) == GTK_RESPONSE_OK)
+                {
+                    
+                }
+            }
+        }
+    }
+#endif
+
     rstto_navigator_clear (window->priv->navigator);
     gtk_widget_set_sensitive(widget, FALSE);
     gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, FALSE);
