@@ -109,6 +109,8 @@ struct _RsttoNavigatorEntry
     gboolean             v_flipped;
     gint                 x_offset;
     gint                 y_offset;
+
+    gboolean             orientation_changed;
 };
 
 
@@ -262,6 +264,64 @@ rstto_navigator_entry_name_compare_func(RsttoNavigatorEntry *a, RsttoNavigatorEn
     return g_strcasecmp(a->info->display_name, b->info->display_name);
 }
 
+
+static void
+rstto_navigator_add_entry_to_history(RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
+{
+
+    /* rstto_debug_print_src_pixbufs(navigator); */
+
+    /* Check if the image is already available in the image-cache */
+    if (g_list_index(navigator->history, entry) != -1)
+    {
+        /* Remove the image from the history, we are going to re-add it at the front of the list later */
+        navigator->history = g_list_remove(navigator->history, entry);
+    }
+    navigator->history = g_list_prepend(navigator->history, entry);
+
+    /* manage history by kicking excess elements out */
+    GList *iter = NULL;
+    guint64 size = 0;
+
+    for (iter = navigator->history; iter != NULL; iter = g_list_next(iter))
+    {
+        RsttoNavigatorEntry *nav_entry = iter->data;
+
+        if (nav_entry)
+        {
+            size += rstto_navigator_entry_get_size(nav_entry);
+            size += sizeof (*nav_entry);
+            size += sizeof (*nav_entry->info);
+
+            if (nav_entry->exif_data);
+                size += sizeof (*nav_entry->exif_data);
+            if (nav_entry->loader)
+                size += sizeof (nav_entry->loader);
+
+            if (size > navigator->max_history)
+            {
+                if (iter != navigator->file_iter)
+                {
+                    rstto_navigator_entry_clear (nav_entry);
+                    iter = g_list_previous(iter);
+                }
+                else
+                {
+                    iter = g_list_previous(iter);
+                    nav_entry = iter->data;
+                    if (nav_entry)
+                    {
+                        rstto_navigator_entry_clear (nav_entry);
+                    }
+                    iter = g_list_next(iter);
+                }
+                navigator->history = g_list_remove(navigator->history, nav_entry);
+            }
+        }
+    }
+}
+
+
 void
 rstto_navigator_guard_history(RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
 {
@@ -297,46 +357,81 @@ rstto_navigator_guard_history(RsttoNavigator *navigator, RsttoNavigatorEntry *en
         }
     }
 
-    /* add image to the cache-history */
-    if (g_list_index(navigator->history, entry) == -1)
-    {
-        navigator->history = g_list_prepend(navigator->history, entry);
-    }
-
-    GList *iter = NULL;
-    guint64 size = 0;
-
-    for (iter = navigator->history; iter != NULL; iter = g_list_next(iter))
-    {
-        RsttoNavigatorEntry *nav_entry = iter->data;
-
-        if (nav_entry)
-        {
-            size += rstto_navigator_entry_get_size(nav_entry);
-
-            if (size > navigator->max_history)
-            {
-                if (iter != navigator->file_iter)
-                {
-                    rstto_navigator_entry_clear (nav_entry);
-                    iter = g_list_previous(iter);
-                }
-                else
-                {
-                    iter = g_list_previous(iter);
-                    nav_entry = iter->data;
-                    if (nav_entry)
-                    {
-                        rstto_navigator_entry_clear (nav_entry);
-                    }
-                    iter = g_list_next(iter);
-                }
-                navigator->history = g_list_remove(navigator->history, nav_entry);
-            }
-        }
-    }
+    rstto_navigator_add_entry_to_history(navigator, entry);
 }
 
+/* for debugging:
+ * iterate through navigator->file_list and print out all
+ * pixbuf entries to see whether they've been loaded or not
+ */
+void
+rstto_debug_print_src_pixbufs (RsttoNavigator *navigator)
+{
+    GList *e = navigator->file_list;
+    int i = 1;
+    printf("src_pixbufs loaded for these image indices: ");
+    while (e != NULL)
+    {
+        RsttoNavigatorEntry *re = e->data;
+        if (re->src_pixbuf != NULL)
+        {
+            printf("%d ", i);
+        }
+        e = g_list_next(e);
+        i++;
+    }
+    printf("\n");
+}
+
+
+/* preloading for improved responsiveness
+ * (at the expense of increased memory usage) 
+ */
+static gboolean
+rstto_navigator_preload_next_img (RsttoNavigator *navigator)
+{
+    GList *next = g_list_next(navigator->file_iter);
+    /* wraparound case */
+    if (next == NULL)
+    {
+        next = navigator->file_list;
+    }
+    if (next != NULL)
+    {
+        RsttoNavigatorEntry *next_entry = next->data;
+        rstto_navigator_entry_load_image(next_entry, FALSE);
+
+        /* add image to the cache-history */
+        rstto_navigator_add_entry_to_history(navigator, next_entry);
+    }
+
+    /* for use with g_timeout_add() */
+    return FALSE;
+}
+
+static gboolean
+rstto_navigator_preload_prev_img (RsttoNavigator *navigator)
+{
+    GList *prev = g_list_previous(navigator->file_iter);
+    /* wraparound case */
+    if (prev == NULL)
+    {
+        prev = g_list_last(navigator->file_list);
+    }
+    if (prev != NULL)
+    {
+        RsttoNavigatorEntry *prev_entry = prev->data;
+        rstto_navigator_entry_load_image(prev_entry, FALSE);
+
+        /* add image to the cache-history */
+        rstto_navigator_add_entry_to_history(navigator, prev_entry);
+    }
+    
+    /* for use with g_timeout_add() */
+    return FALSE;
+}
+
+
 void
 rstto_navigator_jump_first (RsttoNavigator *navigator)
 {
@@ -349,6 +444,13 @@ rstto_navigator_jump_first (RsttoNavigator *navigator)
     if(navigator->file_iter)
     {
         g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED], 0, 0, navigator->file_iter->data, NULL);
+
+        if (navigator->preload)
+        {
+            /* preload in both directions */
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_next_img, navigator);
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_prev_img, navigator);
+        }
     }
 }
 
@@ -372,6 +474,12 @@ rstto_navigator_jump_forward (RsttoNavigator *navigator)
                       g_list_position(navigator->file_list, navigator->file_iter),
                       navigator->file_iter->data,
                       NULL);
+
+        if (navigator->preload)
+        {
+            /* preload forwards */
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_next_img, navigator);
+        }
     }
     else
     {
@@ -405,6 +513,12 @@ rstto_navigator_jump_back (RsttoNavigator *navigator)
                       g_list_position(navigator->file_list, navigator->file_iter),
                       navigator->file_iter->data,
                       NULL);
+
+        if (navigator->preload)
+        {
+            /* preload backwards */
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_prev_img, navigator);
+        }
     }
 }
 
@@ -426,6 +540,13 @@ rstto_navigator_jump_last (RsttoNavigator *navigator)
                       g_list_position(navigator->file_list, navigator->file_iter),
                       navigator->file_iter->data,
                       NULL);
+
+        if (navigator->preload)
+        {
+            /* preload in both directions */
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_next_img, navigator);
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_prev_img, navigator);
+        }
     }
 }
 
@@ -440,17 +561,8 @@ rstto_navigator_set_running (RsttoNavigator *navigator, gboolean running)
             navigator->id = g_timeout_add(navigator->timeout, (GSourceFunc)cb_rstto_navigator_running, navigator);
             if (navigator->preload)
             {
-                GList *next = g_list_next(navigator->file_iter);
-                if (next == NULL)
-                {
-                    next = navigator->file_list;
-                }
-                if (next != NULL)
-                {
-                    RsttoNavigatorEntry *next_entry = next->data;
-                    
-                    rstto_navigator_entry_load_image(next_entry, FALSE);
-                }
+                /* preload forwards */
+                rstto_navigator_preload_next_img(navigator);
             }
         }
     }
@@ -545,9 +657,13 @@ rstto_navigator_remove (RsttoNavigator *navigator, RsttoNavigatorEntry *entry)
             if(!navigator->file_iter)
                 navigator->file_iter = g_list_first(navigator->file_list);
 
-            /* Ehm... an item can exist several times inside the history? */
+            /* An item should not be able to exist several times inside the
+             * history, g_list_remove should suffice here
+             */
             if (navigator->history)
-                navigator->history = g_list_remove_all(navigator->history, entry);
+            {
+                navigator->history = g_list_remove (navigator->history, entry);
+            }
 
             if (navigator->busy == FALSE)
             {
@@ -613,7 +729,6 @@ rstto_navigator_clear (RsttoNavigator *navigator)
     }
     g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED], 0, -1, NULL, NULL);
     g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_REORDERED], 0, NULL);
-    
 }
 
 void
@@ -647,21 +762,14 @@ cb_rstto_navigator_running(RsttoNavigator *navigator)
 
         if (navigator->preload)
         {
-            GList *next = g_list_next(navigator->file_iter);
-            if (next == NULL)
-            {
-                next = navigator->file_list;
-            }
-            if (next != NULL)
-            {
-                RsttoNavigatorEntry *next_entry = next->data;
-                
-                rstto_navigator_entry_load_image(next_entry, FALSE);
-            }
+            /* preload forwards */
+            rstto_navigator_preload_next_img(navigator);
         }
     }
     else
+    {
         navigator->id = 0;
+    }
     return navigator->running;
 }
 
@@ -689,6 +797,7 @@ RsttoNavigatorEntry *
 rstto_navigator_entry_new (RsttoNavigator *navigator, ThunarVfsInfo *info)
 {
     RsttoNavigatorEntry *entry = NULL;
+    ExifEntry *exifentry = NULL;
     gchar *filename = thunar_vfs_path_dup_string(info->path);
     if(filename)
     {
@@ -703,8 +812,11 @@ rstto_navigator_entry_new (RsttoNavigator *navigator, ThunarVfsInfo *info)
             entry->info = info;
             entry->exif_data = exif_data_new_from_file(filename);
             entry->navigator = navigator;
+            entry->animation = NULL;
             
-            ExifEntry *exifentry = exif_data_get_entry(entry->exif_data, EXIF_TAG_ORIENTATION);
+            if (entry->exif_data) {
+                exifentry = exif_data_get_entry(entry->exif_data, EXIF_TAG_ORIENTATION);
+            }
             if (exifentry)
             {
                 gchar *val = g_new0(gchar, 20);
@@ -782,9 +894,13 @@ gboolean
 rstto_navigator_entry_get_flip (RsttoNavigatorEntry *entry, gboolean horizontal)
 {
     if (horizontal)
+    {
         return entry->h_flipped;
+    }
     else
+    {
         return entry->v_flipped;
+    }
 }
 
 static void
@@ -815,6 +931,7 @@ rstto_navigator_entry_clear (RsttoNavigatorEntry *nav_entry)
         g_signal_handlers_disconnect_by_func(nav_entry->loader , cb_rstto_navigator_entry_area_prepared, nav_entry);
         gdk_pixbuf_loader_close(nav_entry->loader, NULL);
     }
+
     if(nav_entry->animation)
     {
         g_object_unref(nav_entry->animation);
@@ -839,6 +956,8 @@ rstto_navigator_entry_free(RsttoNavigatorEntry *nav_entry)
 GdkPixbuf *
 rstto_navigator_entry_get_thumb(RsttoNavigatorEntry *entry, gint size)
 {
+    ThunarVfsPath *home_path = thunar_vfs_path_get_for_home();
+    ThunarVfsPath *thumbnail_path = thunar_vfs_path_relative (home_path, ".thumbnails");
     if(entry->thumb)    
     {
         if(!(gdk_pixbuf_get_width(entry->thumb) == size || gdk_pixbuf_get_height(entry->thumb) == size))
@@ -852,47 +971,55 @@ rstto_navigator_entry_get_thumb(RsttoNavigatorEntry *entry, gint size)
         gchar *thumbnail = thunar_vfs_thumb_factory_lookup_thumbnail(entry->navigator->factory, info);
         if (thumbnail == NULL)
         {
-            GdkPixbuf *pixbuf = thunar_vfs_thumb_factory_generate_thumbnail(entry->navigator->factory, info);
-            if (pixbuf != NULL)
+            if (!thunar_vfs_path_is_ancestor (info->path, thumbnail_path))
             {
-                if (!thunar_vfs_thumb_factory_store_thumbnail(entry->navigator->factory, pixbuf, info, NULL))
+                GdkPixbuf *pixbuf = thunar_vfs_thumb_factory_generate_thumbnail(entry->navigator->factory, info);
+                if (pixbuf != NULL)
                 {
-                    g_critical("Storing thumbnail failed");
-                }
+                    if (!thunar_vfs_thumb_factory_store_thumbnail(entry->navigator->factory, pixbuf, info, NULL))
+                    {
+                        g_critical("Storing thumbnail failed");
+                    }
 
-                gint width = gdk_pixbuf_get_width(pixbuf);
-                gint height = gdk_pixbuf_get_height(pixbuf);
+                    gint width = gdk_pixbuf_get_width(pixbuf);
+                    gint height = gdk_pixbuf_get_height(pixbuf);
 
-                if (width > height)
-                {
-                    entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
-                                                  gdk_pixbuf_get_has_alpha(pixbuf),
-                                                  gdk_pixbuf_get_bits_per_sample(pixbuf),
-                                                  size,
-                                                  height*size/width);
+                    if (width > height)
+                    {
+                        entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
+                                                      gdk_pixbuf_get_has_alpha(pixbuf),
+                                                      gdk_pixbuf_get_bits_per_sample(pixbuf),
+                                                      size,
+                                                      height*size/width);
+                    }
+                    else
+                    {
+                        entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
+                                                      gdk_pixbuf_get_has_alpha(pixbuf),
+                                                      gdk_pixbuf_get_bits_per_sample(pixbuf),
+                                                      width*size/height,
+                                                      size);
+                    }
+                    gdk_pixbuf_scale(pixbuf, entry->thumb,
+                                     0, 0, 
+                                     gdk_pixbuf_get_width(entry->thumb),
+                                     gdk_pixbuf_get_height(entry->thumb),
+                                     0, 0,
+                                     ((gdouble)gdk_pixbuf_get_width(entry->thumb)) / (gdouble)width,
+                                     ((gdouble)gdk_pixbuf_get_height(entry->thumb)) / (gdouble)height,
+                                     GDK_INTERP_BILINEAR);
                 }
                 else
                 {
-                    entry->thumb = gdk_pixbuf_new(GDK_COLORSPACE_RGB,
-                                                  gdk_pixbuf_get_has_alpha(pixbuf),
-                                                  gdk_pixbuf_get_bits_per_sample(pixbuf),
-                                                  width*size/height,
-                                                  size);
+                    thumbnail = thunar_vfs_path_dup_string(info->path);
+                    entry->thumb = gdk_pixbuf_new_from_file_at_scale(thumbnail, size, size, TRUE, NULL);
+                    g_free(thumbnail);
                 }
-                gdk_pixbuf_scale(pixbuf, entry->thumb,
-                                 0, 0, 
-                                 gdk_pixbuf_get_width(entry->thumb),
-                                 gdk_pixbuf_get_height(entry->thumb),
-                                 0, 0,
-                                 ((gdouble)gdk_pixbuf_get_width(entry->thumb)) / (gdouble)width,
-                                 ((gdouble)gdk_pixbuf_get_height(entry->thumb)) / (gdouble)height,
-                                 GDK_INTERP_BILINEAR);
             }
             else
             {
-                thumbnail = thunar_vfs_path_dup_string(info->path);
-                entry->thumb = gdk_pixbuf_new_from_file_at_scale(thumbnail, size, size, TRUE, NULL);
-                g_free(thumbnail);
+                GtkIconTheme *theme = gtk_icon_theme_get_default();
+                entry->thumb = gtk_icon_theme_load_icon (theme, thunar_vfs_mime_info_lookup_icon_name (info->mime_info, theme), size, 0, NULL);
             }
         }
         else
@@ -926,10 +1053,14 @@ rstto_navigator_entry_set_scale (RsttoNavigatorEntry *entry, gdouble scale)
     }
     /* Max scale 1600% */
     if (scale > 16)
+    {
         scale = 16;
+    }
     /* Min scale 5% */
     if (scale < 0.05)
+    {
         scale = 0.05;
+    }
     entry->scale = scale;
 }
 
@@ -954,6 +1085,7 @@ rstto_navigator_entry_set_rotation (RsttoNavigatorEntry *entry, GdkPixbufRotatio
         entry->src_pixbuf = gdk_pixbuf_rotate_simple(pixbuf, (360+(rotation-entry->rotation))%360);
     }
     entry->rotation = rotation;
+    entry->orientation_changed = TRUE;
     g_signal_emit(G_OBJECT(entry->navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ENTRY_MODIFIED], 0, entry, NULL);
 }
 
@@ -1011,6 +1143,12 @@ rstto_navigator_entry_get_size (RsttoNavigatorEntry *entry)
     return size;
 }
 
+gboolean
+rstto_navigator_entry_get_orientation_changed (RsttoNavigatorEntry *entry)
+{
+    return entry->orientation_changed;
+}
+
 gboolean
 rstto_navigator_entry_load_image (RsttoNavigatorEntry *entry, gboolean empty_cache)
 {
@@ -1023,6 +1161,11 @@ rstto_navigator_entry_load_image (RsttoNavigatorEntry *entry, gboolean empty_cac
     }
     if ((entry->loader == NULL) && ((empty_cache == TRUE ) || entry->src_pixbuf == NULL))
     {
+        if (entry->src_pixbuf)
+        {
+            gdk_pixbuf_unref(entry->src_pixbuf);
+            entry->src_pixbuf = NULL;
+        }
         entry->loader = gdk_pixbuf_loader_new();
 
         g_signal_connect(entry->loader, "area-prepared", G_CALLBACK(cb_rstto_navigator_entry_area_prepared), entry);
@@ -1030,6 +1173,7 @@ rstto_navigator_entry_load_image (RsttoNavigatorEntry *entry, gboolean empty_cac
         g_signal_connect(entry->loader, "closed", G_CALLBACK(cb_rstto_navigator_entry_closed), entry);
 
         path = thunar_vfs_path_dup_string(entry->info->path);
+
         entry->io_channel = g_io_channel_new_file(path, "r", NULL);
 
         g_io_channel_set_encoding(entry->io_channel, NULL, NULL);
@@ -1115,6 +1259,8 @@ cb_rstto_navigator_entry_area_prepared (GdkPixbufLoader *loader, RsttoNavigatorE
         entry->src_pixbuf = NULL;
     }
 
+    g_object_ref (entry->animation);
+
     gint time = gdk_pixbuf_animation_iter_get_delay_time(entry->iter);
 
     if (time != -1)
@@ -1138,14 +1284,14 @@ cb_rstto_navigator_entry_area_prepared (GdkPixbufLoader *loader, RsttoNavigatorE
 static void
 cb_rstto_navigator_entry_closed (GdkPixbufLoader *loader, RsttoNavigatorEntry *entry)
 {
+    GdkPixbuf *pixbuf = NULL;
+
     if (entry->src_pixbuf)
     {
         gdk_pixbuf_unref(entry->src_pixbuf);
         entry->src_pixbuf = NULL;
     }
 
-    GdkPixbuf *pixbuf = NULL;
-
     if (entry->iter)
     {
         pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(entry->iter);
@@ -1168,11 +1314,14 @@ cb_rstto_navigator_entry_closed (GdkPixbufLoader *loader, RsttoNavigatorEntry *e
     if (pixbuf != NULL)
     {
         entry->src_pixbuf = gdk_pixbuf_rotate_simple(pixbuf, rstto_navigator_entry_get_rotation(entry));
+        gdk_pixbuf_unref(pixbuf);
+        pixbuf = NULL;
         if (rstto_navigator_entry_get_flip(entry, FALSE))
         {
             pixbuf = entry->src_pixbuf;
             entry->src_pixbuf = gdk_pixbuf_flip(pixbuf, FALSE);
             gdk_pixbuf_unref(pixbuf);
+            pixbuf = NULL;
         }
 
         if (rstto_navigator_entry_get_flip(entry, TRUE))
@@ -1180,6 +1329,7 @@ cb_rstto_navigator_entry_closed (GdkPixbufLoader *loader, RsttoNavigatorEntry *e
             pixbuf = entry->src_pixbuf;
             entry->src_pixbuf = gdk_pixbuf_flip(pixbuf, TRUE);
             gdk_pixbuf_unref(pixbuf);
+            pixbuf = NULL;
         }
         g_signal_emit(G_OBJECT(entry->navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ENTRY_MODIFIED], 0, entry, NULL);
     }
@@ -1340,6 +1490,14 @@ rstto_navigator_entry_select (RsttoNavigatorEntry *entry)
                       navigator->file_iter->data,
                       NULL);
         }
+
+        if (navigator->preload)
+        {
+            /* preload in both directions */
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_next_img, navigator);
+            g_timeout_add(500, (GSourceFunc)rstto_navigator_preload_prev_img, navigator);
+
+        }
     }
 
 }
@@ -1473,7 +1631,7 @@ rstto_navigator_open_file(RsttoNavigator *navigator, const gchar *path, gboolean
     g_free(file_uri);
 
     g_free(file_media);
-    return 0;
+    return TRUE;
 }
 
 gboolean
